/*****************************************************************************
 * Copyright (C) The Apache Software Foundation. All rights reserved.        *
 * ------------------------------------------------------------------------- *
 * This software is published under the terms of the Apache Software License *
 * version 1.1, a copy of which has been included  with this distribution in *
 * the LICENSE file.                                                         *
 *****************************************************************************/
/*****************************************************************************
 * @author <a href="mailto:pankaj_kumar@hp.com">Pankaj Kumar</a>             *
 *         (Hewlett-Packard Company)                                         *
 * Date Originally Created: July 18 - 22, 2001                               *
 * @version CVS $Revision$ $Date$                                            *
 *****************************************************************************/
package org.apache.cocoon.tools.cpp.transformer;

import java.io.FileWriter;
import java.io.IOException;
import java.io.*;
import java.net.*;
import java.util.Date;
import java.util.Map;
import java.util.HashMap;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.cocoon.transformation.*;
import org.apache.avalon.excalibur.pool.Poolable;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 *
 * The <code>CPPTransformer</code> is a class that can be plugged into a Cocoon pipeline
 * to send the SAX events, aftering marking them up with HTML tags, which passes
 * thru this transformer to a CPPServer. The CPPServer
 * creates a CPPViewer window and passes the HTML markup SAX data to it.
 * <br>
 * The CPPServer will be specified in a parameter tag in the sitemap pipeline to the
 * transformer as follows:
 * <p>
 * <pre>
 * &lt;map:transform type="cpp"&gt;
 * &nbsp;&nbsp;&lt;parameter name="name" value="default"/&gt;
 * &nbsp;&nbsp;&lt;parameter name="host" value="localhost"/&gt;
 * &nbsp;&nbsp;&lt;parameter name="port" value="2950"/&gt;
 * &lt;/map:transform&gt;>
 * </pre>
 * </p>
 *
 * <br>
 * This transformation's main purpose is debugging.

 */
public class CPPTransformer extends AbstractTransformer implements Poolable
  {
    /** Wether we are forwarding XML data or not. */
    private boolean canReset=true;

    private String lf = System.getProperty("line.separator", "\n");

    private String hostName = null;
    private int hostPort = 2950;
    private String color = null;
    private String size = null;
    private int indentation = -1;	// Value of -1 implies no line breaks !!
    private boolean initialized = false;
    private String markupBegin = null;
    private String markupEnd = null;

    CPPChannel chan = null;
    private int curIndent = 0;

    /** BEGIN SitemapComponent methods **/

    public void setup(SourceResolver resolver, Map objectModel,
                      String source, Parameters parameters)
            throws ProcessingException, SAXException, IOException
      {
	  String name = parameters.getParameter("name", "default");
      hostName = parameters.getParameter("host", "localhost");
      String portString = parameters.getParameter("port", "2950");
      hostPort = (new Integer(portString)).intValue();

      color = parameters.getParameter("color", "black");
      size = parameters.getParameter("size", "-1");
      String indentationString = parameters.getParameter("indentation", "-1");
      indentation = (new Integer(indentationString)).intValue();

      markupBegin = "<font color=" + color + " size=" + size + ">";
      markupEnd = "</font>";


      getLogger().debug("CPPTransformer.setup()[" + name + "], markupBegin: " + markupBegin + ", indentation: " + indentation);
      chan = (CPPChannel)CPPChannelManager.get(name, hostName, hostPort);
      }

    /** END SitemapComponent methods **/

    /**
     * Receive an object for locating the origin of SAX document events.
     */
    public void setDocumentLocator(Locator locator)
      {
      if (super.contentHandler!=null)
        super.contentHandler.setDocumentLocator(locator);
      }

    /**
     * Receive notification of the beginning of a document.
     */
    public void startDocument() throws SAXException
      {
      if (super.contentHandler!=null)
         super.contentHandler.startDocument();

      this.canReset=false;
      if (!chan.isActive())	// No more processing needed
        return;
      synchronized (chan)
        {
		curIndent = 0;
        Date date = new Date();
        chan.println(markupBegin + "<b>[" + date.toString() + "]Document Start:<br></b>" + markupEnd);
	    }
      }

    /**
     * Receive notification of the end of a document.
     */
    public void endDocument() throws SAXException
      {
      if (super.contentHandler!=null)
        super.contentHandler.endDocument();
      this.canReset=true;
      if (!chan.isActive())	// No more processing needed
        return;
      synchronized (chan)
        {
		curIndent = 0;
        Date date = new Date();
        chan.println(markupBegin + "<b><br>[" + date.toString() + "]Document End.<br></b>" + markupEnd);
	    }
      }

    /**
     * Begin the scope of a prefix-URI Namespace mapping.
     */
    public void startPrefixMapping(String prefix, String uri) throws SAXException
      {
      if (super.contentHandler!=null)
        super.contentHandler.startPrefixMapping(prefix,uri);

      if (!chan.isActive())	// No more processing needed
        return;
      synchronized (chan)
        {
        printIndent(chan);
        chan.println(markupBegin + "&lt;xmlns:" + prefix + "=\"" + uri + "\"&gt;" + markupEnd);
        if (indentation > -1)
          chan.println("<br>");
	    }
      }

    /**
     * End the scope of a prefix-URI mapping.
     */
    public void endPrefixMapping(String prefix) throws SAXException
      {
      if (super.contentHandler!=null)
        super.contentHandler.endPrefixMapping(prefix);

      if (!chan.isActive())	// No more processing needed
        return;
      synchronized (chan)
        {
        printIndent(chan);
        chan.println(markupBegin + "&lt;/xmlns:" + prefix + "&gt;" + markupEnd);
        if (indentation > -1)
          chan.println("<br>");
	    }
      }

    /**
     * Receive notification of the beginning of an element.
     */
    public void startElement(String uri, String loc, String raw, Attributes a) throws SAXException
      {
      if (super.contentHandler!=null)
        super.contentHandler.startElement(uri,loc,raw,a);

      if (!chan.isActive())	// No more processing needed
        return;

      synchronized (chan)
        {
        printIndent(chan);
        curIndent += indentation;
        chan.print(markupBegin + "&lt;" + raw);
        for (int i = 0; i < a.getLength(); i++)
          {
	      chan.print(" " + a.getQName(i) + "=\"" + a.getValue(i) + "\"");
          }
        chan.println("&gt;" + markupEnd);
        if (indentation > -1)
          chan.println("<br>");
	    } //synchronized
      }


    /**
     * Receive notification of the end of an element.
     */
    public void endElement(String uri, String loc, String qname) throws SAXException
      {
      if (super.contentHandler!=null)
        super.contentHandler.endElement(uri,loc,qname);

      if (!chan.isActive())	// No more processing needed
        return;

      synchronized (chan)
        {
        curIndent -= indentation;
        printIndent(chan);
        chan.println(markupBegin + "&lt;/" + qname + "&gt;" + markupEnd);
        if (indentation > -1)
          chan.println("<br>");
	    } //synchronized
    }

    /**
     * Receive notification of character data.
     */
    public void characters(char ch[], int start, int len) throws SAXException
      {
      if (super.contentHandler!=null)
        super.contentHandler.characters(ch,start,len);

      if (!chan.isActive())	// No more processing needed
        return;

      synchronized (chan)
        {
        printIndent(chan);
        chan.println(markupBegin + new String(ch,start,len) + markupEnd);
        if (indentation > -1)
          chan.println("<br>");
	    } //synchronized
      }

    /**
     * Receive notification of ignorable whitespace in element content.
     */
    public void ignorableWhitespace(char ch[], int start, int len)
    throws SAXException {
        if (super.contentHandler!=null) {
            super.contentHandler.ignorableWhitespace(ch,start,len);
        }
    }

    /**
     * Receive notification of a processing instruction.
     */
    public void processingInstruction(String target, String data)
    throws SAXException {
        if (super.contentHandler!=null) {
            super.contentHandler.processingInstruction(target,data);
        }
    }

    /**
     * Receive notification of a skipped entity.
     */
    public void skippedEntity(String name)
    throws SAXException {
        if (super.contentHandler!=null) {
            super.contentHandler.skippedEntity(name);
        }
    }

    /**
     * Report the start of DTD declarations, if any.
     */
    public void startDTD(String name, String publicId, String systemId)
    throws SAXException {
        if (super.lexicalHandler!=null) {
            super.lexicalHandler.startDTD(name,publicId,systemId);
        }
    }

    /**
     * Report the end of DTD declarations.
     */
    public void endDTD()
    throws SAXException {
        if (super.lexicalHandler!=null) {
            super.lexicalHandler.endDTD();
        }
    }

    /**
     * Report the beginning of an entity.
     */
    public void startEntity(String name)
    throws SAXException {
        if (super.lexicalHandler!=null) {
            super.lexicalHandler.startEntity(name);
        }
    }

    /**
     * Report the end of an entity.
     */
    public void endEntity(String name)
    throws SAXException {
        if (super.lexicalHandler!=null) {
            super.lexicalHandler.endEntity(name);
        }
    }

    /**
     * Report the start of a CDATA section.
     */
    public void startCDATA()
    throws SAXException {
        if (super.lexicalHandler!=null) {
            super.lexicalHandler.startCDATA();
        }
    }

    /**
     * Report the end of a CDATA section.
     */
    public void endCDATA()
    throws SAXException {
        if (super.lexicalHandler!=null) {
            super.lexicalHandler.endCDATA();
        }
    }

    /**
     * Report an XML comment anywhere in the document.
     */
    public void comment(char ch[], int start, int len)
    throws SAXException {
        if (super.lexicalHandler!=null) {
            super.lexicalHandler.comment(ch,start,len);
        }
    }

    private void printIndent(CPPChannel chan)
      {
	  for (int i = 0; i < curIndent; i++)
	    chan.print("&nbsp;");
      }
}
